/*
 * Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package monasca.persister.consumer;

import com.google.inject.Inject;
import com.google.inject.assistedinject.Assisted;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.concurrent.ExecutorService;

public class KafkaConsumer<T> {

  private static final Logger logger = LoggerFactory.getLogger(KafkaConsumer.class);

  private ExecutorService executorService;

  private final KafkaConsumerRunnableBasic<T> kafkaConsumerRunnableBasic;
  private final String threadId;

  @Inject
  public KafkaConsumer(
      @Assisted KafkaConsumerRunnableBasic<T> kafkaConsumerRunnableBasic,
      @Assisted String threadId,
      @Assisted ExecutorService executorService) {

    this.kafkaConsumerRunnableBasic = kafkaConsumerRunnableBasic;
    this.threadId = threadId;
    this.executorService = executorService;

  }

  public void start() {

    logger.info("[{}]: start", this.threadId);

    executorService.submit(kafkaConsumerRunnableBasic.setExecutorService(executorService));

  }

  public void stop() {

    logger.info("[{}]: stop", this.threadId);

    kafkaConsumerRunnableBasic.stop();

  }
}
