/*
 * Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
 * implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package monasca.persister.repository.vertica;

import org.skife.jdbi.v2.DBI;
import org.skife.jdbi.v2.Handle;

public class VerticaRepo {
  protected DBI dbi;
  protected Handle handle;

  public VerticaRepo(DBI dbi) {
    this.dbi = dbi;
    this.handle = dbi.open();
    this.handle.execute("SET TIME ZONE TO 'UTC'");
  }

  public VerticaRepo() {}

  public void setDBI(DBI dbi) throws Exception {
    this.dbi = dbi;
    this.handle = dbi.open();
  }
}
